function h = nan_andrewsplot(X,groups,quant,stdize)
// Andrews plot for multivariate data.
// Calling Sequence
// h = nan_andrewsplot(X)
// h = nan_andrewsplot(X,groups)
// h = nan_andrewsplot(X,groups,quant)
// h = nan_andrewsplot(X,groups,quant,stdize)
// Parameters
// X :  Rows of X correspond to observations, columns to variables.
// groups: plots the data in different groups   with different colors, must be a numeric array or character matrix,
// quant: if it is not empty it plots only the median and the    quant and (1-quant) quantiles of f(t) at each value of t.  This is   useful if X contains many observations.
// stdize = 'off' : do nothing
// stdize ='on' : scales each column of X to have  zero mean and unit standard deviation before making the plot.
// stdize = 'pca' : creates an Andrews plot from   the principal component scores of X, in order of decreasing eigenvalue.
// stdize = 'pcastd' : uses the standardized   principal component scores.
// h: handles to the line   objects
// Description
//   nan_andrewsplot(X) creates an Andrews plot of the multivariate data in the
//   matrix X. 
//   
//   An Andrews plot is a tool for visualizing high dimensional data, where
//   each observation is represented by a function, f(t), of a continuous
//   dummy variable, t, over the interval [0,1].  f(t) is defined for the
//   i-th observation in X as
//
//      f(t) = X(i,1)/sqrt(2) + X(i,2)*sin(2*pi*t) + X(i,3)*cos(2*pi*t) + ...
//
//   andrewsplot treats NaNs in X as missing values, and ignores the  corresponding rows. 
//   Examples
//
//      // make a grouped plot of the raw data
//       loadmatfile("demos/data/iris.mat");
//      nan_andrewsplot(meas,species);
//
//      // plot only the median and quartiles of each group
//      nan_andrewsplot(meas,species,.25);
//      
//   Bibliography
//     [1] Gnanadesikan, R. (1977) Methods for Statistical Dara Analysis   of Multivariate Observations, Wiley.
// See also
// nan_parallelcoords


[nargout,nargin]=argn(0);
if nargin < 1
    error('At least one input argument required.');
end
[n,p] = size(X);

if nargin<2 then
   groups=[];
   quant=[];
   stdize='off';
elseif nargin<3 then
      quant=[];
   stdize='off';
elseif nargin<4 then
   stdize='off';
end;


if ~isempty(quant)
    if ~or(type(quant)==[1 5 8]) | ~sum(length(quant))==1 | ~(0 < quant & quant < 1)
        error('The quantile parameter value must be a scalar between 0 and 1.');
    end
    quant = min(quant,1-quant);
end

// Grouping vector was not given, or it is empty to match an empty X.  Fake
// a group index vector.
if isempty(groups)
    gidx = ones(n,1);
    ngroups = 1;
    gnames = [];
// Get the group index for each observation
else
    [gidx,gnames] = nan_grp2idx(groups);
    ngroups = size(gnames,'*');
    if length(gidx) ~= n
        error(' The ''group'' parameter value must have one row for each row of X.');
    end
end
obsinds = 1:n;
// Remove NaNs from data
nans = find(isnan(gidx) | or(isnan(X),2));
if ~isempty(nans)
    X(nans,:) = [];
    gidx(nans) = [];
    obsinds(nans) = [];
    n = size(X,1);
end

// Transform data if requested
if type(stdize)==10
    select convstr(stdize)
    case 'off' then
        // leave X alone
        
    // Standardize each coordinate to unit variance
    case 'on' then
        if n > 1
            X = (X - repmat(nan_mean(X,1),n,1))./ repmat(nan_std(X,[],1),n,1);
        elseif n == 1
            X = zeros(X); // prevent divide by zero
        else
            // leave empty X alone
        end
        
    // Transform the data to PC scores
    case 'pca' then // 'pcastd'} 
        if ~isempty(X)
            [dum,X,variance] = princomp(X); 
            
        end
      case 'pcastd' then // 'pcastd'} 
        if ~isempty(X)
            [dum,X,variance] = princomp(X);
                // Leave columns of X with zero variance alone.
                tol = max(n,p)*%eps;
                variance(variance<tol) = 1;
                X = X ./ repmat(sqrt(variance(:)'),n,1);
        end
        
    else
        error(' The ''standardize'' parameter must be ''off'', ''on'', ''pca'', or ''pcastd''.');
    end
else
    error(' The ''standardize'' parameter value be a string.');
end

// Compute the terms for the curves.  The constant term goes with the first
// column of X, the lowest freq sin with the 2nd, the lowest freq cos with
// the third, and so on.
t = 0:.001:1;
omega = 2*%pi*(1:floor(p/2));
omegaSin = omega(1:floor(p/2));
omegaCos = omega(1:floor((p-1)/2));
A = [ones(1,length(t))./sqrt(2); sin(omegaSin' * t); cos(omegaCos' * t)];
F = X(:,[1 (2:2:p) (3:2:p)])*A;


scf();
cax=gca();

colorlist=["b","r","g","k","y","c","m"];
ncolors = size(colorlist,'*');

hh = list();
lgndh = [];;
for grp = 1:ngroups
    colorNr = (pmodulo(grp-1,ncolors)+1);
    mbrs = find(gidx == grp);
    
    // Make an empty plot if no data.
    if isempty(mbrs)
        plot(t,%nan*ones(size(t,1),size(t,2)), 'LineStyle','-', 'Color',colorlist(colorNr));
        
    // Plot the individual observations, or the median and the upper and
    // lower ALPHA-quantiles of the data.  Any unused input args are passed
    // to plot as graphics properties.
    elseif isempty(quant)
        // Plot rows of F against t.  If a group has p members, plot would
        // try to use columns of F, prevent that by always using F'.
        plot(t,F(mbrs,:)', 'LineStyle','-', 'Color',colorlist(colorNr));
        
       
        hh(grp) = gce();
    else
        if length(mbrs) == 1
            per = [.5 .5 .5];
            q = repmat(F(mbrs,:),3,1); // no dim arg for prctile
        else
            per = [.50 quant 1-quant];
            q = prctile(F(mbrs,:), 100*per);
        end
        plot(t,q(1,:), 'LineStyle','-', 'Color',colorlist(colorNr));
        hh(grp) = gce();
        plot(t,q(2:3,:), 'LineStyle',':', 'Color',colorlist(colorNr));
        //hh((grp-1)*2 + (2)) = gce();
       
        
    end
    
    // Save line handles for the legend if the data are grouped
     if ~isempty(groups) & ~isempty(mbrs)
         if isempty(lgndh)
             lgndh=hh(grp).children(1);
         else
              lgndh= [lgndh; hh(grp).children(1)];
         end;
     end
end

if nargout > 0
    h = hh;
end

// Label the axes
xlabel('t'); ylabel('f(t)');


// If the data are grouped, put up a legend
if ~isempty(groups)
    tt = 1:size(lgndh,'*');  // use find because length may not match
    legend(lgndh,gnames(tt,:));
end

endfunction


