function nan_boxplot(varargin)
// Draw a box-and-whiskers plot for data provided as column vectors.
//
// Calling Sequence
//  nan_boxplot(y1[,y2,...])
//
// Parameters
//  y1,y2,... : vectors and/or matrixes with data as column vectors (of varying length) 
//
// Description
//  A boxplot (also known as a box-and-whisker plot is a way of 
//  graphically depicting groups of numerical data through their five-number summaries 
//  (the smallest observation, lower quartile (Q1), median, upper quartile (Q3), and 
//  largest observation). A boxplot may also indicate which observations, if any, 
//  might be considered outliers. The boxplot was invented in 1977 by the 
//  American statistician John Tukey.
//
//  For each data series a box is drawn to indicate the position of the lower and upper 
//  quartile of the data. The box has a centre line indicating the median of the data.
//  Straigh center lines (the whiskers) above and below the box indicates the 
//  maximum and minimum values in the data set (except for outliers). 
//  Outliers are defined as any points larger than Q3 + 1.5*IQR or lower then
//  Q1 - 1.5*IQR, where IQR is the inter quartile range defined as IQR=Q3-Q1.
//  Outliers are plotted as individual '*'. 
//
//  Boxplots can be useful to display differences between populations without making 
//  any assumptions of the underlying statistical distribution. The spacings between 
//  the different parts of the box help indicate the degree of dispersion (spread) and 
//  skewness in the data, and identify outliers. 
//
// Examples
//  nan_boxplot([rand(10,5);5*rand(5,5)-2.5]) 
//  nan_boxplot() // demo and help
//
// Bibliography
//  http://en.wikipedia.org/wiki/Box_plot
//
// Authors
//  T. Pettersen (2008)

// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.

if argn(2)==0 then
  nan_boxplot([rand(10,5);5*rand(5,5)-2.5]);
  xtitle('boxplot() draws a Box and Whiskers plot','Data series no.','Data range');
  help nan_boxplot
end

X=0; wd=1/4; style='b-';
set(gcf(),'immediate_drawing','off');
for i=1:argn(2),
  Y=varargin(i);
  if size(Y,'r')==1 then error('Input data should be given as column vectors'); end
  for j=1:size(Y,'c'),
    X=X+1;
    Q=quart(Y(:,j)); Min=min(Y(:,j)); Max=max(Y(:,j));
    plot([X-wd;X+wd;X+wd;X-wd;X-wd;],[Q(1);Q(1);Q(3);Q(3);Q(1)],style);  // the quartile box
    plot([X-wd;X+wd],[Q(2);Q(2)],style); // the median
    IQR=Q(3)-Q(1);  // the inter quartile range = Q3 - Q1
    outliers=Y(find(Y(:,j)>Q(3)+1.5*IQR),j);
    outliers=[outliers;Y(find(Y(:,j)<Q(1)-1.5*IQR),j)];
    if ~isempty(outliers) then 
      plot(X*ones(outliers),outliers,'*'); // the outliers - if any
      large=find(outliers>Q(3));
      if large then
        whiskers=max(Y(find(Y(:,j)<min(outliers(large))),j));
      else
        whiskers=Max;
      end
      small=find(outliers<Q(1));
      if small then
        whiskers=[min(Y(find(Y(:,j)>max(outliers(small))),j));whiskers];
      else
        whiskers=[Min;whiskers];
      end
    else
      whiskers=[Min;Max];
    end
    plot([X;X],[whiskers(1);Q(1)],style);   // the lower whisker
    plot([X;X],[whiskers(2);Q(3)],style);   // the upper whiskers
  end
end
set(gcf(),'immediate_drawing','on');
endfunction
