function [cm,gn] = nan_confusionmat(g,ghat,order)
//  Confusion matrix for classification algorithms.
//  Calling Sequence
//  CM = nan_confusionmat(G,GHAT)
//  CM = nan_confusionmat(G,GHAT,ORDER)
//  [CM, GORDER] = nan_confusionmat(G, GHAT)
//  Parameters
//  G: known group labels
//  GHAT: predicted group labels
//  CM : confusion matrix determined   by the known group labels G and the predicted group labels GHAT
//  ORDER: if used the order of rows (and columns) of the confusion matrix is specified by ORDER.  ORDER is a
//    vector containing the same elements as in G and GHAT
//   GORDER: order of group labels   for rows and columns of CM. GORDER has the same type as G and GHAT.
//   Description
//    The confusion matrix CM(I,J) represents the count of instances
//    whose known group labels are group J and whose predicted group labels
//    are group I.
//    
//                   known Classes
//                   
//              ___|_____|______| 
//              
//    Predicted  ' |     |      |
//    
//      Classes    |     |      |
//    
//     confusionmat treats NaNs, empty strings or 'undefined'
//    values in G or GHAT as missing values, and the corresponding
//    observations are not counted.
//
//
//   Examples
//      // Compute the resubstitution confusion matrix for applying CLASSIFY
//      // on Fisher iris data.
//       loadmatfile("demos/data/iris.mat");
//      x = meas;
//      y = species;
//      yhat = nan_classify(x,x,y);
//      [cm,order] = nan_confusionmat(y,yhat);
//
//   See also 
//   nan_grp2idx


 [nargout,nargin]=argn(0);
if nargin < 2
    error('At least two inputs are required.');
elseif nargin==2
  order=[];
end

gClass = typeof(g);
if ~(gClass==typeof(ghat))
    error('G and GHAT need to be the same type.');
end

if ~or(type(g)==[1 5 8]) & ~type(g)==4   & ~iscell(g)  & ~type(g)==10
    error('G and GHAT can only be a categorical, numeric, or logical vector; a cell vector of strings; or a character matrix.');
end

if type(g)==10
    if length(size(g)) > 2 | length(size(ghat)) > 2
        error('G and GHAT must be 2D if they are character arrays.');
    end
    g = g(:);
    ghat = ghat(:);
elseif ~isvector(g) | ~isvector(ghat) 
    error('G and GHAT need to be vectors or 2D character arrays.');
else
    g = g(:);
    ghat = ghat(:);
end


if size(g,1) ~= size(ghat,1)
    error('G and GHAT need to have same number of rows');
end




if ~isempty(order)
    if type(order)==10
        if length(size(order)) > 2
            error('ORDER must be 2D if it is a character array.');
        end
        order = order(:);
    elseif ~isvector(order)
        error( 'ORDER must be a vector or a 2D array of character.');
    end


        if or(type(g)==[1 5 8])
            if ~or(type(order)==[1 5 8])
                error('ORDER must be the same class as G and GHAT..');
            end
            if or(isnan(order))
                error('ORDER can''t have NaN values.');
            end

        elseif type(g)==4
            if type(order)==4
                //OK. do nothing
            elseif or(type(order)==[1 5 8])
                if or(isnan(order))
                    error('ORDER can''t have NaN values.');
                end

                order = (order)~=0;
                
            else
                error('ORDER must be the same class as G and GHAT.');
            end

        elseif iscell(g)
            if ~iscell(order)
                error( 'ORDER should contain all the labels appearing in G or GHAT.');
            end
            if or((''==(order)))
                error('ORDER can''t have empty strings.');
            end
        end

        try
            uorder = unique(order);
        catch 
            error('ORDER must have a UNIQUE method.');
        end

        if max(size(uorder)) < max(size(order))
            error('ORDER can''t contain duplicated values.');
        end

        order = order(:);
end


gLen = size(g,1);
[idx,gn,gnn] =nan_grp2idx([g;ghat]);
gidx = idx(1:gLen);
ghatidx = idx(gLen+1:gLen*2);

//ignore NaN values in GIDX and GHATIDX
nanrows = isnan(gidx) | isnan(ghatidx);
if or(nanrows)
    gidx(nanrows) = [];
    ghatidx(nanrows) = [];
end

gLen = size(gidx,1);
gnLen =max(size(gn));

cm = nan_accumarray([gidx,ghatidx], ones(gLen,1),[gnLen, gnLen])';

if ~isempty(order)
    //get the map from the default order to the given order
    
    [hasAllLabel,map] = nan_ismember(gnn,order);

        if ~and(hasAllLabel)
            error('ORDER should contain all the labels appearing in G or GHAT.');
        end
        orderLen = max(size(order));
        cm2 = zeros(orderLen, orderLen);
        cm2(map,map) = cm(:,:);
        cm = cm2;

    if nargout > 1
        gn = order;
        // convert gn to categorical if g is categorical
        if (gClass=='string')
            gn = string(gn);
        elseif (gClass=="boolean")
            gn = gn~=0;
        end
    end
else
//convert gn to the same type as g
gn=gnn;

end

endfunction

