function [Nout,Cout] = nan_ecdfhist(F,X,param1,param2,opt_plot)
//   Create histogram from ecdf output.
//   Calling Sequence
//   N = nan_ecdfhist(F,X)
//   N = nan_ecdfhist(F,X,M)
//   N = nan_ecdfhist(F,X,C)
//   [N,Cout] = nan_ecdfhist(...) 
//   nan_ecdfhist(...,'plot') produces a histogram bar plot of   the results.
//   Parameters
//   F: vector of empirical cdf values
//   X: vector of evaluation points
//   M: uses M bins instead of 10
//   C: vector with centers of bins
//   N: heights of   histogram bars for 10 equally spaced bins
//   Cout: position of the bin centers
// Description
//    The bar heights of the histogram are computed   from the increases in the empirical cdf, 
//    and are normalized so the area
//   for each bar represents the probability for the corresponding interval.
//   If F is computed from a censored sample, the total probability may be less
//   than 1.  In contrast, HIST produces bars whose heights are bin counts and
//   whose areas do not represent probabilities.
//
//
//   Examples
//        y = grand(50,1,'exp',10); 
//       [F,X] = nan_ecdf(y);
//       nan_ecdfhist(F,X,'plot');
//       // Superimpose a plot of the known true pdf
//       xx = 0:.1:max(y); yy = exp(-xx/10)/10; plot(xx,yy,'g-');
//
//   See also
//   nan_ecdf
//   nan_histc

[nargout,nargin]=argn(0);


in_par=['F','X','param1','param2','opt_plot'];
in_par_min=2;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;
//error(nargchk(2,Inf,nargin,'struct'));

// Accept an axes as the first arg, and plot into it.
if nargin>0 & sum(length(F))==1 & typeof(F)=="handle"
    // Get the axes, and shift the remaining args down by one.
    cax = F;
    F = X;
    if nargin > 2
        X = param1;
        if nargin > 3
            param1 = param2;
            param2=[];
        end;
    end
    nargs = nargin - 1;
else
    cax = [];
    nargs = nargin;
end

if nargs<2
    error('Requires both F and X.');
elseif nargs <3 then
     param1=[];
     param2=[];
elseif nargs < 4 then
     param2=[];
end

// Inputs should look like they came from ecdf.
if type(F)==10 | type(X)==10
    error('Input arguments must be numeric.')
end
if length(F)~=length(F) | length(X)~=length(X)
    error('Both F and X must be vectors produced by the ecdf function.');
elseif length(F)<2 | length(X)<2
    error( 'Both F and X must have two or more elements.');
elseif or(diff(X)<0)
    error('X must be non-decreasing.');
end

// For convenience, accept F as a survivor function by converting to cdf
wassurv = (F(1)==1);
if wassurv
    F = 1-F;
end
diffF = diff(F);
if or(diff(F)<0)
    if wassurv
        error( 'The survivor function F must be non-increasing')
    else
        error('The cdf F must be non-decreasing')
    end
end

// Chop off extra 1st X value
X = X(2:$);

// Get bin edges, counts, etc.
if isempty(param1) & isempty(param2)
    [P,edges,C,useCenters,wng] = statgetbins(X,diffF);
elseif ~isempty(param1) & isempty(param2)
    [P,edges,C,useCenters,wng] = statgetbins(X,diffF,param1);
else
    [P,edges,C,useCenters,wng] = statgetbins(X,diffF,param1,param2);
end
nbins = length(edges)-1;

// Convert to bar heights
binwidth = diff(edges);
N = (P ./ binwidth);

// Plot if no outputs requested
if plotting
    if isempty(cax), cax = gca(); end
    if useCenters
        // Make sure the first and last bars extend to full width of the data.
        //h = bar(cax,C,N,[edges(1) edges($)],'hist');
        bar(cax,C,N);
        cax.auto_ticks(1)='on'
    else
        bar(cax,edges(1:$-1), N);h=gce();
         cax.auto_ticks(1)='on'

    end
    //set(cax,'xtickmode','auto');
else
    Nout = N;
    if nargout>=2
        Cout = C;
        if ~isempty(wng)
            warning(wng);
        end
    end
end

endfunction




function [bincount,edges,C,useCenters,wng] = statgetbins(x,freqs,binspec,specval)
//STATGETBINS Get bin edges from bin specification
//   This is a utility used by statistics functions to process 'bins',
//   'edges', and 'centers' arguments.

[nargout,nargin]=argn(0);

if nargin<2
   error('Wrong number of input arguments.');
end

wng = ''; 
lo = double(min(x(:)));
hi = double(max(x(:)));

// May have bin count, bin centers, or bin edges.  Default is 10 bins.  If
// third arg is a string, it specifies how to interpret the fourth arg as a
// bin specification.  If third arg is a scalar, assume nbins.  Otherwise,
// assume ctrs.
useCenters = %t;
if nargin < 3
    binspec = 'nbins';
    nbins = 10;
else
    if type(binspec)==10
        select binspec
        case 'ctrs'  // ecdfhist(F,X,'ctrs',C)
            ctrs = specval(:)'; // force a row
        case 'edges' // ecdfhist(F,X,'edges',E)
            edges = specval(:)'; // force a row
            useCenters = false;
        case 'nbins' // ecdfhist(F,X,'nbins',M)
            nbins = specval;
        else
            error('You must specify ''nbins'', ''ctrs'', or ''edges''.');
        end
    elseif length(binspec) == 1  // ecdfhist(F,X,M)
        nbins = binspec;
        binspec = 'nbins';
    else                         // ecdfhist(F,X,C)
        ctrs = binspec(:)'; // force a row
        binspec = 'ctrs';
    end
end

// Translate the bin spec into bin centers and edges.
select binspec
case 'nbins'
    // If the bin count is specified:
    if isempty(x)
        lo = 0;
        hi = 1;
    end
    if lo == hi
        lo = lo - floor(nbins/2) - 0.5;
        hi = hi + ceil(nbins/2) - 0.5;
    end
    binwidth = (hi - lo) ./ nbins;
    edges = lo + binwidth*(0:nbins);
    edges(length(edges)) = hi;
    C = edges(1:$-1) + binwidth/2;
case 'ctrs'
    // Bin centers specified.  Create edges midway between the centers, and
    // symmetric about the extreme centers.
    C = ctrs;
    binwidth = diff(C);
    if or(binwidth<=0)
        error('The vector C of bin centers must be increasing.');
    end
    binwidth = [binwidth binwidth($)];
    edges = [C(1)-binwidth(1)/2 C+binwidth/2];
    // Make sure the edges span the data.
    if ~isempty(lo)
       edges(1) = min(edges(1),lo);
       edges($) = max(edges($),hi);
    end
case 'edges'
    // Bin edges are specified.
    binwidth = diff(edges);
    if or(binwidth<=0)
        error('The vector E of bin edges must be increasing.');
    end
    if nargout > 2
        // Create centers that have the edges midway between them.
        C = mtlb_zeros(size(binwidth));
        C(1) = edges(1) + binwidth(1)/2;
        for j = 2:length(C)
            C(j) = 2*edges(j) - C(j-1);
        end
        // It may not be possible to find centers for which the edges are
        // midpoints.  Caller may wish to warn if that's the case.
        if or(C <= edges(1:$-1)) | abs(C($) - (edges($)-binwidth($)/2)) > 1000*%eps;
            wng = 'Cannot compute centers that are consistent with EDGES.';
            C = edges(1:$-1) + binwidth/2;
        end
    end
    // The edges may or may not span the data.
    if ~isempty(x) & ((lo < edges(1)) | (edges($) < hi))
        warning('The vector E of bin edges does not span the range of X.');
    end
end

// Update bin widths for internal bins
nbins = length(edges) - 1;

if isempty(x)
    binnum = x;
elseif useCenters
    // Shift bins so the internal is ( ] instead of [ ).
    edges = edges + %eps;
    // Map each jump location to a bin number. -Inf accounts for the above
    // shift, +Inf keeps things out of histc's degenerate rightmost bin.
    [ignore,binnum] = nan_histc(x,[-%inf edges(2:$-1) %inf]);
else
    // Map each jump location to a bin number.  Only jumps that are within
    // the closed interval from edges(1) to edges(end) get counted.
    [ignore,binnum] = nan_histc(x,edges);
    // Merge histc's degenerate rightmost bin with the last "real" bin, and
    // ignore anything out of range.
    binnum(binnum==nbins+1) = nbins;
end
        
if or(binnum==0)
    freqs(binnum==0) = [];
    binnum(binnum==0) = [];
end

// Compute the probability for each bin
binnum = binnum(:);
bincount = nan_accumarray([mtlb_ones(size(binnum)),binnum],freqs,[1,nbins]);
endfunction

