function [h,ax,BigAx] = nan_gplotmatrix(x,y,g,clr,sym,siz,doleg,dispopt,xnam,ynam)
//  Scatter plot matrix with grouping variable.
//  Calling Sequence
//  nan_gplotmatrix(X,Y,G)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ,DOLEG)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ,DOLEG,DISPOPT)
//  nan_gplotmatrix(X,Y,G,CLR,SYM,SIZ,DOLEG,DISPOPT,XNAM,YNAM)
//  nan_gplotmatrix(X,[],G,...)
//  [H,AX,BigAx] = nan_gplotmatrix(...)
//  Parameters
//  X: P by M matrix
//  Y: if Y is a P by N Matrix, N by M scatter plots will be created. If y is empty X vs X will be plotted
//  G: grouping  variable that determines the marker and color assigned to each point in   each matrix
//  CLR : string of color specifications. Default is 'bgrcmyk'
//  SYM : string of marker specifications. Default is '.'
//  SIZ : marker size to use  for all plots 
//  DOLEG : 'on' legends are plotted. 'off' means no legens will be plotted at all
//  DISPOPT : lets you control how to  fill the diagonals in a plot of X vs. X. Set DISPOPT to 'none' to  leave them blank, 'hist' (default) to plot histograms, or 'variable' to   write the variable names.
//  XNAM : names of the X variables, must be a   character array or cell array of strings of the appropriate dimension.
//  YNAM : names of the Y variables, must be a   character array or cell array of strings of the appropriate dimension.
//  H: handle of the plotted points
//  AX: handle of the subaxes
//  BigAx : handle to the figure
//  Description
//   nan_gplotmatrix creates a matrix of scatter plots of the columns of
//   X against the columns of Y, grouped by G.  If Y is [], scatter plots of X vs X will be plotted.
//   
//   If the number of columns of X is M and the number of columns of Y is N, then N by M scatter plots will be created.
//
//   Examples
//     loadmatfile("demos/data/iris.mat");
//      varNames = {'Sepal Length','Sepal Width','Petal Length','Petal Width'};
//      nan_gplotmatrix(meas,[],species,'bgrcm',[],[],'on','hist',varNames');
//
//   See also
//   nan_grpstats
//   nan_gscatter
//   nan_plotmatrix


[nargout,nargin]=argn(0);
if nargin<1
    error('Requires one Parameter');
end;

nin = nargin;

if (nin < 2), y = []; end
if isempty(y) // gplotmatrix(x)
  rows = size(x,2); cols = rows;
  y = x;
  XvsX = %t;
else // gplotmatrix(x,y)
  rows = size(y,2); cols = size(x,2);
  XvsX = %f;
end
if (nin > 2) & ~isempty(g)
   [g,gn] = nan_mgrp2idx(g,size(x,1),',');
   ng = max(g);
else
   g = [];
   gn = [];
   ng = 1;
end

// Default colors, markers, etc.
if (nin < 4) | isempty(clr), clr = 'bgrcmyk'; end
if (nin < 5) | isempty(sym), sym = '.'; end
if (nin < 6), siz = []; end
if (nin < 7) | isempty(doleg), doleg = 'on'; end
if (nin < 8) | isempty(dispopt), dispopt = 'h'; end
if (nin < 9) | isempty(xnam)
   xnam = {};
else
   if type(xnam)==10 & (size(xnam,1)==cols)
       xnam = cellstr(xnam);
   elseif typeof(xnam)=="ce" & (length(xnam)==cols)
      // ok
   else
      error('XNAM must be a character array or cell array of strings, with\none variable name for each column of X.');
   end
end
if (XvsX)
   ynam = xnam;
elseif (nin < 10) | isempty(ynam)
   ynam = {};
else
   if type(ynam)==10 & (size(ynam,1)==rows)
       ynam = cellstr(ynam);
   elseif typeof(ynam)=="ce" & (length(ynam)==rows)
      // ok
   else
      error('YNAM must be a character array or cell array of strings, with\none variable name for each column of Y.');
   end
end

// What should go into the plot matrix?
doleg = (doleg== 'on') & (~XvsX | (rows>1)) & ~isempty(gn);
dohist = XvsX & (part(dispopt,1)=='h');
donames = (XvsX & (part(dispopt,1)=='v'));

// Don't plot anything if either x or y is empty
if isempty(rows) | isempty(cols),
   if nargout>0, h = []; ax = []; BigAx = []; end
   return
end

if length(size(x))>2 | length(size(y))>2
   error('X and Y must be 2-D.');
end
if size(x,1)~=size(y,1)
  error( 'X and Y must have the same length.');
end
if (~isempty(g)) & (length(g) ~= size(x,1)),
  error( 'There must be one value of G for each row of X.');
end

// Create/find BigAx and make it invisible
clf();
BigAx = gca();

//hold_state = ishold;
//set(BigAx,'Visible','off','color','none')

if (isempty(siz))
   siz = repmat(3, size(sym));
   if ~isempty(strindex(sym,'.')),
//       units = get(BigAx,'units');
//       set(BigAx,'units','pixels');
      pos = [get(BigAx.parent,'figure_position'),get(BigAx.parent,'figure_size')];
//       set(BigAx,'units',units);
      siz(sym=='.') = max(0,min(15,round(1*min(pos(3:4))/size(x,1)/max(rows,cols))));
   end
end
if donames
  offset=30;
else
  offset=50;
end
// Create and plot into axes
//ax2filled = ones(max(rows,cols),1)*%f;
 pos = [get(BigAx.parent,'figure_position'),get(BigAx.parent,'figure_size')];
width = (pos(3)-offset)/cols;
height = (pos(4)-offset)/rows;
space = .02; // 2 percent space between axes
pos(1:2) = pos(1:2) + space*[width height];

[m,n] = size(y); 
xlim = repmat(cat(3,mtlb_min(x,[],1),mtlb_max(x,[],1)),[rows 1 1]);
ylim = repmat(cat(3,mtlb_min(y,[],1)',mtlb_max(y,[],1)'),[1 cols 1]);
ax=list();ax2=list();hh=list();hhdiag=list();

for i=rows:-1:1,
   for j=cols:-1:1,
      axPos = [(j-1)*width+offset (i-1)*height   width*(1-space) height*(1-space)];
      ax(rows*(i-1)+j) = newaxes();//axes('Position',axPos, 'visible', 'on', 'Box','on');
      ax(rows*(i-1)+j).axes_bounds=axPos./[pos(3),pos(4),pos(3),pos(4)];
      if ((i==j) & XvsX)
         if (dohist)
            //histax = axes('Position',axPos);
            //ax2(j) = ax(rows*(i-1)+j);
            //ax2filled(j) = %t;
            //[nn,xx] = hist(reshape(y(:,i,:),[m k]));
            [nn,xx] = nan_hist(y(:,i));
            //hhdiag(i) = bar(xx,nn,'hist');
            bar(xx,nn);
            hhdiag(i)=gce();
            //set(histax, 'YAxisLocation', 'right',  'Visible','off', 'XTick',[], 'YTick',[],  'XGrid','off', 'YGrid','off',  'XTickLabel','', 'YTickLabel','');
            ax(rows*(i-1)+j).auto_ticks=["on","on","on"];
            //axis tight;
            ax(rows*(i-1)+j).tight_limits="on";
            //xlim(i,j,:) = get(gca,'xlim');
            xlim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,1);
            // Don't set ylim, so histogram heights won't affect y limits.
         end
      else
         //hhij = iscatter(reshape(x(:,j,:),[m k]), reshape(y(:,i,:),[m k]), g, clr, sym, siz);
         hhij = iscatter(x(:,j),y(:,i), g, clr, sym, siz);
         hh(rows*(i-1)+j)=hhij;
         //hh(i,j,1:length(hhij)) = hhij;
         //axis tight;
         ax(rows*(i-1)+j).tight_limits="on";
         ylim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,2);
         xlim(i,j,:) = ax(rows*(i-1)+j).data_bounds(:,1);
         //ylim(i,j,:) = get(gca,'ylim');
         //xlim(i,j,:) = get(gca,'xlim');


      end
      //set(ax(i,j),'xlimmode','auto', 'ylimmode','auto', 'xgrid','off', 'ygrid','off')
   end
end

// Fill in histogram handles
if XvsX & dohist
    for i=1:rows
        hh(rows*(i-1)+i) = hhdiag(i);
    end
end

xlimmin = mtlb_min(xlim(:,:,1),[],1); xlimmax = mtlb_max(xlim(:,:,2),[],1);
ylimmin = mtlb_min(ylim(:,:,1),[],2); ylimmax = mtlb_max(ylim(:,:,2),[],2);

// Set all the limits of a row or column to be the same and leave 
// just a 5// gap between data and axes.
inset = .05;
// for i=1:rows,
//   set(ax(i,1),'ylim',[ylimmin(i,1) ylimmax(i,1)])
//   dy = diff(get(ax(i,1),'ylim'))*inset;
//   set(ax(i,:),'ylim',[ylimmin(i,1)-dy ylimmax(i,1)+dy])
// end
// for j=1:cols,
//   set(ax(1,j),'xlim',[xlimmin(1,j) xlimmax(1,j)])
//   dx = diff(get(ax(1,j),'xlim'))*inset;
//   set(ax(:,j),'xlim',[xlimmin(1,j)-dx xlimmax(1,j)+dx])
//   if ax2filled(j)
//      set(ax2(j),'xlim',[xlimmin(1,j)-dx xlimmax(1,j)+dx])
//   end
// end

 for i=1:rows,
  for j=1:cols
     //set(ax(i,1),'ylim',[ylimmin(i,1) ylimmax(i,1)])
    
      ax(rows*(i-1)+j).data_bounds(:,1)=[xlimmin(1,j) xlimmax(1,j)]';
      dx = diff(ax(rows*(i-1)+j).data_bounds(:,1))*inset;
//     set(ax(:,j),'xlim',[xlimmin(1,j)-dx(j) xlimmax(1,j)+dx(j)])
     ax(rows*(i-1)+j).data_bounds(:,1)=[xlimmin(1,j)-dx xlimmax(1,j)+dx]';
//      if (ax2filled(j)
//         hh(rows*(i-1)+j).data_bounds(:,1)=[xlimmin(1,j)-dx xlimmax(1,j)+dx]';
//      end
        ax(rows*(i-1)+j).data_bounds(:,2)=[ylimmin(i,1) ylimmax(i,1)]';
        dy = diff(ax(rows*(i-1)+j).data_bounds(:,2))*inset;
        //set(ax(i,:),'ylim',[ylimmin(i,1)-dy ylimmax(i,1)+dy])
       ax(rows*(i-1)+j).data_bounds(:,2)=[ylimmin(i,1)-dy ylimmax(i,1)+dy]';
     
  end;
 end



// Label plots one way or the other
if (donames & ~isempty(xnam))
   for j=1:cols
      //set(gcf,'CurrentAx',ax(j,j));
      sca(ax(rows*(j-1)+j))
     // xstring((  xlimmin(1,j)+xlimmax(1,j))/2, (ylimmin(j,1)+ylimmax(j,1))/2,    xnam(j).entries);//, 'HorizontalAlignment','center',        'VerticalAlignment','middle');
      xstring((  xlimmin(1,j)), (ylimmin(j,1)+ylimmax(j,1))/2,    xnam(j).entries);//, 'HorizontalAlignment','center',        
   end
else
   if ~isempty(xnam)
      for j=1:cols, xlabel(ax(rows*(rows-1)+j),xnam(j).entries); end
   end
   if ~isempty(ynam)
      for i=1:rows, ylabel(ax(rows*(i-1)+1),ynam(i).entries); end
   end
end

// Ticks and labels on outer plots only
 for i=1:rows-1,
  for j=2:cols
  ax(rows*(i-1)+j).x_ticks.labels=[''];
  ax(rows*(i-1)+j).y_ticks.labels=[''];
   ax(rows*(i-1)+j).margins=[0.01 0.01 0.01 0.01];
end
  
end
for j=2:cols
 ax(rows*(rows-1)+j).y_ticks.labels=[''];
 ax(rows*(rows-1)+j).margins=[0.01 0.01 0.01 0.01];
end;
 for i=1:rows-1,
 ax(rows*(i-1)+1).x_ticks.labels=[''];
ax(rows*(i-1)+1).margins=[0.01 0.01 0.01 0.01];
end;
ax(rows*(rows-1)+1).margins=[0.01 0.01 0.01 0.01];
// set(ax(1:rows-1,:),'xticklabel','')
// set(ax(:,2:cols),'yticklabel','')
//set(BigAx,'XTick',get(ax(rows,1),'xtick'),'YTick',get(ax(rows,1),'ytick'),   'userdata',ax,'tag','PlotMatrixBigAx')

// Create legend if requested; base it on the top right plot
if (doleg)
   //gn = gn(nan_ismember(1:size(gn,1),g),:);
   legend(ax(rows*(1-1)+cols),gn);
end

// Make BigAx the CurrentAxes
//set(gcf,'CurrentAx',BigAx)
sca(BigAx);
// if ~hold_state,
//    set(gcf,'NextPlot','replace')
// end

// Also set Title and X/YLabel visibility to on and strings to empty
//set([get(BigAx,'Title'); get(BigAx,'XLabel'); get(BigAx,'YLabel')], 'String','','Visible','on');

if nargout~=0,
  h = hh;
//   if or(ax2filled)
//      ax = [ax; ax2(:)'];
//   end
end
endfunction

function h = iscatter(x,y,i,c,m,msize)
//ISCATTER Scatter plot grouped by index vector.
//   ISCATTER(X,Y,I,C,M,msize) displays a scatter plot of X vs. Y grouped
//   by the index vector I.  
//
//   No error checking.  Use GSCATTER instead.
//
//   See also GSCATTER, GPLOTMATRIX.


[nargout,nargin]=argn(0);
xcols = size(x,2);
ycols = size(y,2);
ncols = max(xcols,ycols);

ni = max(i); // number of groups	
if (isempty(ni))
   i = ones(size(x,1),1);
   ni = 1;
end

nm = length(m);
ns = length(msize);
if type(c)==10 & isvector(c)
    c = c(:);
end
nc = length(c);

// Now draw the plot
for j=1:ni
   ii = (i == j);
   nii = sum(ii);
   if nii==0
       // degenerate case, create lines with NaN data
       for k=1:ncols
           //hh(j,k) = line(%nan,%nan,  'LineStyle','none', 'Color', part(c,(1+mod(j-1,nc),:)),    'Marker', part(m,(1+mod(j-1,nm))), 'MarkerSize', msize(1+mod(j-1,ns)));
            //plot(%nan,%nan,  'LineStyle','none', 'Color', part(c,(1+mod(j-1,nc))),  'Marker', part(m,(1+mod(j-1,nm))), 'MarkerSize', msize(1+mod(j-1,ns)));
            plot(%nan,%nan  ,[part(c,(1+pmodulo(j-1,nc)))+ part(m,(1+pmodulo(j-1,nm)))]);
	   hh(j,k)=gce();
	   if ~isempty(msize)	
		    hh(j,k).children(1).mark_size=msize(1+pmodulo(j-1,ns));
	   end
       end
   elseif nii==1 & xcols>1 & ycols>1
       // degenerate case, avoid plotting two row vectors in one line
       for k=1:ncols
           //hh(j,k) = line(x(ni,k),y(ni,k),  'LineStyle','none', 'Color', c(1+mod(j-1,nc),:), 'Marker', m(1+mod(j-1,nm)), 'MarkerSize', msize(1+mod(j-1,ns)));
           //plot(x(ni,k),y(ni,k),  'LineStyle','none', 'Color', part(c,(1+mod(j-1,nc))), 'Marker', part(m,(1+mod(j-1,nm))), 'MarkerSize', msize(1+mod(j-1,ns)));
           plot(x(ni,k),y(ni,k)  ,[part(c,(1+pmodulo(j-1,nc)))+ part(m,(1+pmodulo(j-1,nm)))]);
	   hh(j,k)=gce();
	   if ~isempty(msize)	
		    hh(j,k).children(1).mark_size=msize(1+pmodulo(j-1,ns));
	   end
       end
   else
       // normal case, create lines for each column pair at once
       //hh(j,:) = line(x(ii,:), y(ii,:), 'LineStyle','none', 'Color', c(1+mod(j-1,nc),:),  'Marker', m(1+mod(j-1,nm)), 'MarkerSize', msize(1+mod(j-1,ns)));
       //plot(x(ii,:), y(ii,:), 'LineStyle','none', 'Color', part(c,(1+pmodulo(j-1,nc))),  'Marker', part(m,(1+mod(j-1,nm))), 'MarkerSize', msize(1+mod(j-1,ns)));
       plot(x(ii,:), y(ii,:),[part(c,(1+pmodulo(j-1,nc)))+ part(m,(1+pmodulo(j-1,nm)))]);
       hh(j)=gce();
       if ~isempty(msize)	
                hh(j).children(1).mark_size=msize(1+pmodulo(j-1,ns));
	end
   end
end

// Return the handles if desired.  They are arranged so that even if X
// or Y is a matrix, the first ni elements of hh(:) represent different
// groups, so they are suitable for use in drawing a legend.
if (nargout>0)
   h = hh(:);
end
endfunction

