function [varargout] = nan_grpstats(x,groups,whichstats,varargin)
// Summary statistics by group.
// Calling Sequence
// MEANS = nan_grpstats(X,GROUP)
// nan_grpstats(X,GROUP,ALPHA) displays a plot of the means versus index with
//   100(1 - ALPHA)%  confidence intervals around each mean.
// [MEANS,SEM,COUNTS,GNAME] = nan_grpstats(X,GROUP)
// [A,B,...] = nan_grpstats(X,GROUP,WHICHSTATS)
// [...] = nan_grpstats(X,GROUP,WHICHSTATS,ALPHA)
// Parameters
// X: matrix of observations
// GROUP: grouping variable defined  as a vector, string array, or cell array of strings.
//   MEANS: means of each column of X by GROUP
//   SEM: standard error  of the mean
//   COUNTS:  number of elements in each group
//   GNAME: name of each group
//   ALPHA: specifies the confidence  level as 100(1-ALPHA)% for the 'meanci' and 'predci' options.
//   WHICHSTATS:a  list of single function handles or names (strings)
//   A,B:  statistics specified by WHICHSTATS, The number of outputs [A,B,...] must match the
//   number function handles and names in WHICHSTATS.
//   
//  Description
//   grpstats computes groupwise summary statistics, for data in a matrix or
//   dataset array.
//   
//   GRPSTATS treats NaNs as missing values, and removes them.
//
//  Names in WHICHSTATS can  be chosen from among the following:
//
//      'mean'     mean
//      'sem'      standard error of the mean
//      'numel'    count, or number of elements
//      'gname'    group name
//      'std'      standard deviation
//      'var'      variance
//      'min'      minimum
//      'max'      maximum
//      'range'    maximum - minimum
//      'meanci'   95% confidence interval for the mean
//      'predci'   95% prediction interval for a new observation
//
// Functions in WHICHTSTATS must accept a matrix and give out one values for each columns.
//
//   Examples
//      loadmatfile("demos/data/iris.mat");
//      [m,p,g] = nan_grpstats(meas,species,list('mean','predci','gname'))
//      n = size(m,1)
//      plot2d((1:n)',m)
//      nan_errorbar(ones(1,4) .*. (1:n)',m,p(:,:,2)-m,p(:,:,2)-m)
//      h=gca();
//      h.data_bounds(:,1)=[0.9;3.1];
//      title('95% prediction intervals for mean iris size by species')
//      scf()
//      nan_grpstats(meas,species,0.05)
//
//   See also
//   nan_gscatter
//   nan_grp2idx


[nargout,nargin]=argn(0);
if (nargin<1)
   error('GRPSTATS requires at least one argument.')
end
if length(size(x))>2 | ~(or(type(x)==[1,5,8]) | type(x)==4)
    error( 'X must be a numeric or logical vector or matrix');
elseif isvector(x)
    x = x(:);
end
[rows,cols] = size(x);

doplot = %f;
varNames=[];
// Distinguish name/value pairs from grpstats(x,group,whichstats,alpha)
if nargin==4
    arg4 = varargin(1);
    if nargin==4 & (or(type(arg4)==[1,4,8]) & sum(length(arg4))==1)
        alpha = arg4;
    elseif nargin==4 & or(type(whichstats)==[1,5,8]) & sum(length(whichstats))==1 & type(arg4)==10
        alpha = whichstats;
        whichstats = [];
        doplot = %t;
        varNames=arg4;

    end


// Recognize plotting syntax with alpha in 3rd position
elseif nargin==3
    if or(type(whichstats)==[1,5,8]) & sum(length(whichstats))==1
        alpha = whichstats;
        whichstats = [];
        doplot = %t;
    else
        alpha = 0.05;
    end
    
elseif nargin<=2
    alpha = 0.05;   // used in nested functions
    whichstats = [];
end

if alpha<=0 | alpha>=1
    error('ALPHA must be a number larger than 0 and smaller than 1.');
end


deff("[t]=mean1(x)","t=mean(x,1)");
deff("[t]=sem1(x)","t=nan_std(x,0,1) / sqrt(size(x,1))");
deff("[t]=std1(x)","t=nan_std(x,0,1)");
deff("[t]=var1(x)","t=var(x,0,1)");
deff("[t]=min1(x)","t=empty2NaN(mtlb_min(x,[],1));");
deff("[t]=max1(x)","t=empty2NaN(mtlb_max(x,[],1));");
deff("[t]=range1(x)","t=empty2NaN(nan_range(x,1));");
deff("[t]=numel1(x)","t=size(x,1)");
// Get list of statistics functions to call
if length(whichstats)==0
    // Default list
    whichstats = list(mean1,sem1,numel1, 'gname');
    if doplot
        minargs = 3;
    else
        minargs = 1;
    end
   for i=max(minargs,nargout)+1:length(whichstats)
       whichstats(i)=null();
    end
else
    if ~typeof(whichstats)=="list"
        whichstats = list(whichstats);
    end

    // Convert keywords to function handles
    for j=1:length(whichstats)
        hfun = whichstats(j);
        if type(hfun)==10
            select(hfun)
              case 'mean' then  hfun = mean1
              case 'sem' then   hfun = sem1
              case 'std' then   hfun = std1
              case 'var' then   hfun = var1
              case 'min' then   hfun = min1
              case 'max' then   hfun = max1
              case 'range' then hfun = range1
              case 'numel' then hfun = numel1
              case 'meanci' then hfun = meanci
              case 'predci' then hfun = predci
              //otherwise, may be a function name or 'gname'
            end
            whichstats(j) = hfun;
         elseif typeof(hfun)=="function"
            select(hfun)
              case mean then  hfun = mean1,
              case sem then   hfun = sem1,
              case nan_std then   hfun = std1,
              case nan_var then   hfun = var1,
              //case min then   hfun = min1,
              //case max then   hfun = max1,
              case range then hfun = range1,
              //case numel then hfun = numel1,
              //otherwise, may be a function name or 'gname'
            end
            whichstats(j) = hfun;

        end
    end
    
    // Warn if they won't get some of what's listed in whichstats; they will get an
    // error if not enough is listed in whichstats
    if max(1,nargout) < length(whichstats)
        warning(sprintf('GRPSTATS called with %d output arguments to compute %d\n summary statistics.',nargout,length(whichstats)));
    end
end

// Get grouping variable information
if (nargin<2) | isempty(groups)
   groups = ones(rows,1);
end
[groups,glabel,groupname,multigroup,ngroups] = nan_mgrp2idx(groups,rows);
if length(groups) ~= rows
    error('Must have one GROUP for each row of X.');
end

// Collect group information
cgroups = list();
for gnum = 1:ngroups
    cgroups(gnum) = find(groups==gnum);
end

nfuns = length(whichstats);
//varargout = cell(1,max(1,nfuns));

for nfun = 1:nfuns
    hfun = whichstats(nfun);   // get function handle or name
    if (hfun=='gname')
        // special case for gname, not applied separately to each column
        varargout(nfun) = groupname;
        
    else

    // Should we try to apply the function to an entire matrix or just a column?
    trymatrix = (cols~=1) & ~or(isnan(x(:)));

    // Test the function to see what we get
    if isempty(cgroups)
        rowidx = [];
    else
        rowidx = cgroups(1);
    end

    if trymatrix
        // Attempt to call the function on a data matrix
        try
            t = hfun(x(rowidx,:));
            if size(t,2)~=cols
                trymatrix = %f;
            end
        catch
            trymatrix = %f;
        end
    end
    if trymatrix
        // Success, put results for this group into an array
        nstatvals = size(t,1);
        t1 = matrix(t',[1,cols,nstatvals]);   // 1st dim for groups
        z = repmat(t1,[ngroups,1,1]);          // one per group
        tsize = [nstatvals,cols];
    else    
        // Call the function on one column
        if size(x,2)>=1
            y = x(rowidx,1);
        else
            y = x(rowidx,[]);
        end
//         if ngroups > 0
//             t = tryeval(hfun,y(~isnan(y),:),glabel{1});
//         else
//             t = tryeval(hfun,y(~isnan(y),:));
//         end
        t = hfun(y(~isnan(y),:));
        nstatvals = size(t,1);
        t1 = matrix(t,[1,1,nstatvals]);       // dims 1-2 for group,col
        z = repmat(t1,[ngroups,cols,1]);       // one per group and col
        tsize = size(t);
        if ngroups>0 & cols>0
            for colnum = 2:cols
                // Now do the rest of the columns
                y = x(rowidx,colnum);
                //z(1,colnum,:) = tryeval(hfun,y(~isnan(y),:),glabel{1},tsize);
                 if length(size(z))==3
                   //z(1,colnum,:) = hfun(y(~isnan(y),:));
                   tmp=  hfun(y(~isnan(y),:));
		    for k=1:size(z(1,colnum,:),3)
		    z(1,colnum,k)=tmp(k,:);
		    end
                 else
                   z(1,colnum) = hfun(y(~isnan(y),:));
                 end
            end
        
      end
    end

    // Now do the rest of the groups
    for gnum = 2:ngroups
        idx = cgroups(gnum);

        if trymatrix
            //z(gnum,:,:) = tryeval(hfun,x(idx,:),glabel{gnum},tsize)';
            if length(size(z))==3
              tmp= hfun(x(idx,:));
              for k=1:size(z(gnum,:,:),3)
               z(gnum,:,k)=tmp(k,:);
              end
              //z(gnum,:,:) = hfun(x(idx,:));
            else
               z(gnum,:) = hfun(x(idx,:));
            end
        else
            for colnum = 1:cols
                y = x(idx,colnum);
                //z(gnum,colnum,:) = tryeval(hfun,y(~isnan(y),:),glabel{gnum},tsize);
               if length(size(z))==3
                 //z(gnum,colnum,:) = hfun(y(~isnan(y),:));
                 tmp= hfun(y(~isnan(y),:));
                 for k=1:size(z(gnum,colnum,:),3)
                    z(gnum,colnum,k)=tmp(k,:);
                 end
               else
                 z(gnum,colnum) = hfun(y(~isnan(y),:));
               end
            end
        end
    end
    
    // Special case:  don't add 3rd dimension of there is just one column
    if cols==1
        z = matrix(z,ngroups,nstatvals);
    end
    varargout(nfun) = z;

  end;
end

if doplot
   means = varargout(1);
   sems = varargout(2);
   counts = varargout(3);
   p = ones(size(counts,1),size(counts,2))*(1 - alpha/2);
   xd = repmat((1:ngroups)',1,cols);
   plot2d(xd,means);
   errbar(xd,means,cdft("T",counts-1,p,1-p) .* sems,cdft("T",counts-1,p,1-p) .* sems);
   h=gca();
   //set(h,'Marker','o','MarkerSize',2);
   //set(gca,'Xlim',[0.5 ngroups+0.5],'Xtick',(1:ngroups));
    if type(glabel)==10
    x_ticks=h.x_ticks;
    x_ticks.locations=(1:ngroups)';
    x_ticks.labels=glabel(:);
    h.x_ticks=x_ticks;
    end
   h.data_bounds(:,1)=[0.5; ngroups+0.5];
   h.data_bounds(:,2)=[min(means-cdft("T",counts-1,p,1-p) .* sems);max(means+cdft("T",counts-1,p,1-p) .* sems)];
   xlabel('Group');
   ylabel('Mean');
   if ~isempty(varNames)
      legend(varNames);
   end
   if (multigroup)
      // Turn off tick labels and axis label
     // set(gca, 'XTickLabel','','UserData',size(groupname,2));
      xlabel('');
      //ylim = get(gca, 'YLim');
      ylim=h.data_bounds(:,2);
      // Place multi-line text approximately where tick labels belong
      for j=1:ngroups
       //  text(j,ylim(1),glabel{j,1},'HorizontalAlignment','center',  'VerticalAlignment','top', 'UserData','xtick');
       xstring(j,ylim(1),glabel(j));
      end

      // Resize function will position text more accurately
      //set(gcf, 'ResizeFcn', @resizefcn, 'Interruptible','off');
      //doresize(gcf);
   else
      //set(gca, 'XTickLabel',glabel);
   end
   title('Means and Confidence Intervals for Each Group');
   //set(gca, 'YGrid', 'on');
   xgrid(2);
end
endfunction
// Nested functions below here; they use alpha from caller
    function ci = meanci(y) // m,s,n,d are local variables
    n = size(y,1);
    m = nan_mean(y,1);
    s = nan_std(y,0,1) / sqrt(n);
    d = s * -cdft("T",max(0,n-1),alpha/2,1-(alpha/2) );
    ci = [m-d; m+d];
    endfunction

    // ----------------------------
    function ci = predci(y) // m,s,n,d are local variables
    n = size(y,1);
    m = nan_mean(y,1);
    s = nan_std(y,0,1) * sqrt(1 + 1/n);
    d = s * -cdft("T",max(0,n-1),alpha/2,1-(alpha/2) );
    ci = [m-d; m+d];
    endfunction

    // ----------------------------
    function m = empty2NaN(m) // convert 0xm empty to NaN(1,m)
    if size(m,1) == 0
        m = ones(1,size(m,2))*%nan;
    end
    endfunction





