function [no,xo] = nan_hist(y,x,param1,opt_plot)
//   Histogram.
//   Calling Sequence
//   N = nan_hist(Y)
//   N = nan_hist(Y,M)
//   N = nan_hist(Y,X)
//   nan_hist(...,'plot')  produces a histogram bar plot of   the results.
//   nan_hist(AX,...) plots into AX instead of GCA.
//   [N,X] = nan_hist(...)
//   Parameters
//   Y: input vector or matrix. If Y is a   matrix, hist works down the columns.
//   M: number of equally spaced containers in which Y is sorted [default: 10]
//   X: X is a vector which specifies  centers of each container. The first bin includes  data between -inf and the first center and the last bin   includes data between the last bin and inf. Note: Use HISTC if
//   it is more natural to specify bin edges instead. 
//   N: number of elements in each container
//   X: position of the bin centers in X.
//  Examples
//  x = -4:0.1:4;
//  y = rand(10000,1,'normal');
//  nan_hist(y,x,'plot') 
//   See also 
//    nan_histc


[nargout,nargin]=argn(0);
if (nargin == 0),
 error('At least 1 parameter required');
end;

in_par=['y','x','param1','opt_plot'];
in_par_min=1;
plotting=0;
if (nargin>in_par_min)
   if (type(evstr(in_par(nargin)))==10)
     if convstr(evstr(in_par(nargin)))=="plot",   plotting=1; end; 
     clear evstr(in_par(nargin));
     nargin=nargin-1;
   end;
end;
clear in_par;

// Accept an axes as the first arg, and plot into it.
if nargin>0 & sum(length(y))==1 & typeof(y)=="handle"
    // Get the axes, and shift the remaining args down by one.
    cax = y;
    y = x;
    if nargin > 2
        x = param1;
    end
    nargs = nargin - 1;
else
    cax = [];
    nargs = nargin;
end

if nargs == 1
    x = 10;
end

if isvector(y), y = y(:); end
if (~or(type(x)==[1 5 8]) | ~or(type(y)==[1 5 8]))
    error( 'Input arguments must be numeric.')
end

// Cache the vector used to specify how bins are created
N = x;
    
if isempty(y),
    if length(x) == 1,
       x = 1:double(x);
    end
    nn = mtlb_zeros(size(x)); // No elements to count
    //  Set miny, maxy for call to bar below.
    miny = [];
    maxy = [];
    edges = [-%inf %inf];
else
    //  Ignore NaN when computing miny and maxy.
    ind = ~isnan(y);
    miny = min(y(ind));
    maxy = max(y(ind));
    //  miny, maxy are empty only if all entries in y are NaNs.  In this case,
    //  max and min would return NaN, thus we set miny and maxy accordingly.
    if (isempty(miny))
      miny = %nan;
      maxy = %nan;
    end
    
    if length(x) == 1
    	  if miny == maxy,
    		  miny = miny - floor(x/2) - 0.5; 
    		  maxy = maxy + ceil(x/2) - 0.5;
     	  end
        binwidth = (maxy - miny) ./ x;
        xx = miny + binwidth*(0:x);
        xx(length(xx)) = maxy;
        x = xx(1:length(xx)-1) + binwidth/2;
    else
        xx = x(:)';
        binwidth = [diff(xx) 0];
        xx = [xx(1)-binwidth(1)/2 xx+binwidth/2];
        xx(1) = min(xx(1),miny);
        xx($) = max(xx($),maxy);
    end
    // Shift bins so the interval is ( ] instead of [ ).
    xx = full(real(xx)); y = full(real(y)); // For compatibility
    bins = xx + %eps;
    edges = [-%inf bins];
    nn = nan_histc(y,edges,1);
    edges(2:$) = xx;    // remove shift
    
    // Combine first bin with 2nd bin and last bin with next to last bin
    nn(2,:) = nn(2,:)+nn(1,:);
    nn($-1,:) = nn($-1,:)+nn($,:);
    nn = nn(2:$-1,:);
    edges(2) = [];
    edges($) = %inf;
end


    if isvector(y) & ~isempty(y) // Return row vectors if possible.
        no = nn';
        xo = x;
    else
        no = nn;
        xo = x';
    end
if plotting
  if ~isempty(cax)
     //bar(x,nn,[miny maxy],'hist');
    bar(cax,x,nn);
     cax.auto_ticks(1)='on';
  else
     //bar(x,nn,[miny maxy]);
     bar(x,nn);
     cax=gca(); cax.auto_ticks(1)='on';
  end
end

endfunction
