function h = nan_parallelcoords(X,varargin)
// Parallel coordinates plot for multivariate data.
// Calling Sequence
//  nan_parallelcoords(AX,...) plots into the axes with handle AX.
//  nan_parallelcoords(X)
//  nan_parallelcoords(X,group)
//  nan_parallelcoords(X,group,alpha)
//  nan_parallelcoords(X,group,alpha,labels)
//  nan_parallelcoords(X,group,alpha,labels,stdize)
// H = nan_parallelcoords(X, ...)
// Parameters
// AX: handle
// group: plots the data in different   groups with different colors. group is a string matrix.
// alpha: if  not empty the the median and the  ALPHA and (1-ALPHA) quantiles of f(t) at each value of t is plotted
// labels: labels the coordinate tick marks
// stdize = 'off' : do nothing
// stdize ='on' : scales each column of X to have  zero mean and unit standard deviation before making the plot.
// stdize = 'pca' : creates an Andrews plot from   the principal component scores of X, in order of decreasing eigenvalue.
// stdize = 'pcastd' : uses the standardized   principal component scores.
// // h: handles to the line   objects
// Description
// 
//   parallelcoords(X) creates a parallel coordinates plot of the
//   multivariate data in the N-by-P matrix X.  Rows of X correspond to
//   observations, columns to variables.  A parallel coordinates plot is a
//   tool for visualizing high dimensional data, where each observation is
//   represented by the sequence of its coordinate values plotted against
//   their coordinate indices.  
//   
//   parallelcoords treats NaNs in X as missing
//   values, and those coordinate values are not plotted.
//
//   Examples
//
//      // make a grouped plot of the raw data
//     loadmatfile("demos/data/iris.mat");
//      labs = {'Sepal Length','Sepal Width','Petal Length','Petal Width'};
//      nan_parallelcoords(meas, species, [],labs);
//
//      // plot only the median and quartiles of each group
//      nan_parallelcoords(meas, species, 0.25,labs);
//
//   See also 
//   nan_andrewsplot
//   Bibliography
//     [1] Gnanadesikan, R. (1977) Methods for Statistical Dara Analysis    of Multivariate Observations, Wiley.


[nargout,nargin]=argn(0);

if sum(length(X))==1 & typeof(X)=="handle"
    ax = X;

    X = varargin(1);
    varargin(1) = [];
else
    ax = [];
end

if nargin < 1+length(ax)
    error('At least one input argument required.');
end
[n,p] = size(X);

// Process input parameter name/value pairs, assume unrecognized ones are
// graphics properties for PLOT.
groups=[];
quantil=[];
tcklabs=[];
stdize='off';


if nargin==2 then
groups=varargin(1);
quantil=[];
tcklabs=[];
stdize='off';
elseif nargin==3
groups=varargin(1);
quantil=varargin(2);
tcklabs=[];
stdize='off';
elseif nargin==4
groups=varargin(1);
quantil=varargin(2);
tcklabs=varargin(3);
stdize='off';
elseif nargin==5
groups=varargin(1);
quantil=varargin(2);
tcklabs=varargin(3);
stdize=varargin(4);
end;

if ~isempty(quantil)
    if ~or(type(quantil)==[1 4 8]) | ~sum(length(quantil))==1 | ~(0 < quantil & quantil < 1)
        error( 'The ''quantile'' parameter value must be a scalar between 0 and 1.');
    end
    quantil = min(quantil,1-quantil);
end

// Grouping vector was not given, or it is empty to match an empty X.  Fake
// a group index vector.
if isempty(groups)
    gidx = ones(n,1);
    ngroups = 1;
    gnames = [];
// Get the group index for each observation.
else
    [gidx,gnames] = nan_grp2idx(groups);
    ngroups = size(gnames,'*');
    if length(gidx) ~= n
        error( 'The ''group'' parameter value must have one row for each row of X.');
    end
end

dfltTckLabs = isempty(tcklabs);
if dfltTckLabs
    tcklabs = string(1:p);
elseif type(tcklabs)==10 & size(tcklabs,'*') == p
    // Ok.
elseif iscell(tcklabs) & length(tcklabs) == p
    // Ok.
else
    error('The ''labels'' parameter value must be a character array or a cell array of strings with one label for each column of X.');
end


origrows = 1:n;
if isempty(quantil)
    // Cache a copy of original X for datatip use, but only if individual
    // observations will be plotted.
    origX = X;
    origQuantiles = [];
else
    // Compute quantiles for each group.
    origX = [];

    origQuantiles = list();
    for grp = 1:ngroups
        mbrs = find(gidx == grp);
        if length(mbrs) == 1
            q = repmat(X(mbrs,:),3,1); // No dim arg for prctile.
        else
            q = nan_prctile(X(mbrs,:), 100*[.50 quantil 1-quantil]);
        end
        origQuantiles(grp) = q;
    end
end

// Transform data if requested.
dopca = %f;
rescaled = %t;
if type(stdize)==10
    select convstr(stdize)
    case 'off' then
        // Leave X alone.
        rescaled = %f;
    // Standardize each coordinate to unit variance.
    case 'on' then
        if n > 1
            X = (X - repmat(nan_mean(X),n,1))./ repmat(nan_std(X),n,1);
        elseif n == 1
            X(~isnan(X)) = 0; // Prevent divide by zero, preserve NaNs.
        else
            // Leave empty X alone.
        end

    // Transform the data to PC scores.
    case 'pca' then
        // Remove NaNs from the data before PCA.
        nans = find(isnan(gidx) | or(isnan(X),2));
        if ~isempty(nans) 
            X(nans,:) = [];
            gidx(nans) = [];
            origrows(nans) = [];
            n = size(X,1);
        end
        if ~isempty(X)
            [dum,X,variance] = princomp(X); 
        end
        dopca = %t;
     case 'pcastd' then
        // Remove NaNs from the data before PCA.
        nans = find(isnan(gidx) | or(isnan(X),2));
        if ~isempty(nans) 
            X(nans,:) = [];
            gidx(nans) = [];
            origrows(nans) = [];
            n = size(X,1);
        end
        if ~isempty(X)
            [dum,X,variance] = princomp(X); 
                // Leave columns of X with zero variance alone.
                tol = max(n,p)*%eps;
                variance(variance<tol) = 1;
                X = X ./ repmat(sqrt(variance(:)'),n,1);
        end
        dopca = %t;


    else
        error('The ''standardize'' parameter must be ''off'', ''on'', ''pca'', or ''pcastd''.');
    end
else
    error( 'The ''standardize'' parameter value be a string.');
end

icoord = 1:p;

//cax = newplot(ax);
cax = gca();
colorlist=["b","r","g","k","y","c","m"];
ncolors = size(colorlist,'*');
hh = list();


lgndh=[];
for grp = 1:ngroups
    colorNr = (pmodulo(grp-1,ncolors)+1);
    mbrs = find(gidx == grp);

    // Make an empty plot if no data.
    if isempty(mbrs)
        plot(icoord,%nan*ones(size(icoord,1),size(icoord,2)), 'LineStyle','-', 'Color',colorlist(colorNr));

    // Plot the individual observations, or the median and the upper and
    // lower ALPHA-quantiles of the data.  Any unused input args are passed
    // to plot as graphics properties.
    elseif isempty(quantil)
        // Plot rows of X against icoord.  If a group has p members, plot 
        // would try to use columns of X, prevent that by always using X'.

        plot(icoord,X(mbrs,:)', 'LineStyle','-', 'Color',colorlist(colorNr));
        // Save line handles ordered by observation, one per row of X.
        hh(grp) = gce();
    else
        if length(mbrs) == 1
            per = [.5 .5 .5];
            q = repmat(X(mbrs,:),3,1); // No dim arg for prctile.
        else
            per = [.50 quantil 1-quantil];
            q = nan_prctile(X(mbrs,:), 100*per);
        end
        plot(icoord,q(1,:), 'LineStyle','-', 'Color',colorlist(colorNr));
        hh(grp) = gce();
        plot(icoord,q(2:3,:), 'LineStyle',':', 'Color',colorlist(colorNr));
        hh((grp-1)*2 + (2)) = gce();
    end

    // Save line handles for the legend if the data are grouped.
    if ~isempty(groups) & ~isempty(mbrs)
         if isempty(lgndh)
             lgndh=hh(grp).children(1);
         else
              lgndh= [lgndh; hh(grp).children(1)];
         end;
    end
end

if nargout > 0
    h = hh;
end

// Label the axes
    if dopca
        if dfltTckLabs, xlabel('Principal Component'); end
        ylabel('PC Score');
    else
        if dfltTckLabs, xlabel('Coordinate'); end
        ylabel('Coordinate Value');
    end
    x_ticks=cax.x_ticks;
    x_ticks.locations=icoord(:);
    x_ticks.labels=tcklabs(:);
    cax.x_ticks=x_ticks;

// If the data are grouped, put up a legend
if ~isempty(groups)
    tt = 1:size(lgndh,'*');  // use find because length may not match
    legend(lgndh,gnames(tt,:));
end

endfunction




